import 'package:flutter/material.dart';
import './route_complete_mixin.dart';

class RouteAniManage {
  //  页面上的route与对应的路由动画结束处理事件数组对应map
  Map<Route, List<RouteAniCompleteMixin>> _routeAniCompleteHandlerMap = {};
  //  页面route与页面路由动画处理事件的映射
  Map<Route, AnimationStatusListener> _routeAniStatusHandlerMap = {};

  //  给指定的路由添加路由动画结束后的处理事件
  addRouteAniCompleteHandler (BuildContext context, RouteAniCompleteMixin handler) {
    ModalRoute route = ModalRoute.of(context);
    if (_routeAniCompleteHandlerMap[route] == null) {
      _routeAniCompleteHandlerMap[route] = [];
    }
    _routeAniCompleteHandlerMap[route].add(handler);
  }

  //  给指定页面的路由动画添加事件监听
  addRouteListener (BuildContext context) {
    ModalRoute route = ModalRoute.of(context);
    AnimationStatusListener routeAniStatusHandler = (AnimationStatus status) {
      _routeAniStatusChangeHandler(status, route);
    };
    _routeAniStatusHandlerMap[route] = routeAniStatusHandler;
    route.controller.addStatusListener(routeAniStatusHandler);
  }

  //  页面路由动画监听事件
  _routeAniStatusChangeHandler (AnimationStatus status, ModalRoute route) {
    if (status == AnimationStatus.completed) {
      List<RouteAniCompleteMixin> handlers = _routeAniCompleteHandlerMap[route];
      if (handlers != null) {
        handlers.forEach((RouteAniCompleteMixin item) {
          item.onRouteAniComplete();
        });
      }
      removeRouteRelativeHandler(route);
    } else if (status == AnimationStatus.dismissed) {
//      removeRouteRelativeHandler(route);
    }
  }

  //  移除指定页面的路由动画监听事件
  removeRouteRelativeHandler (ModalRoute route) {
    print('removeRouteRelativeHandler');
    if (_routeAniStatusHandlerMap[route] != null) {
      route.controller.removeStatusListener(_routeAniStatusHandlerMap[route]);
    }
    if (_routeAniCompleteHandlerMap.containsKey(route)) {
      _routeAniCompleteHandlerMap.remove(route);
    }
  }
}